@@ -151,8 +151,13 @@ class MechModel:
         X_train = afreg.transform(X_train)
         X_test = afreg.transform(X_test)
 
+        X.columns = X.columns.astype('str')
+        X_train.columns = X_train.columns.astype('str')
+        X_test.columns = X_test.columns.astype('str')
+
         self.autofeat = afreg
 
+
         #----------------------------------------------------------------------
         model = RandomForestRegressor(
                     n_estimators=200,
@@ -171,6 +176,7 @@ class MechModel:
         Y_test = Y_test.reshape(-1, 1)
 
         #----------------------------------------------------------------------
+
         model.fit(X, Y.values.ravel())
 
         y_pred = model.predict(X).reshape(-1, 1)
