@@ -1,78 +1,32 @@
+from pyexpat import model
 import pandas as pd
 import pymssql
+import cx_Oracle as cx
+
+from .queries import *
 from pandas import read_sql_query
 
-# to config
-columns2targets = {
-    'FPT': ['FPT'],
-    'FVS': ['FVS'],
-    'FOYD': ['FOYD'],
-    'FOS': ['FOS'],
-    'FPTP': ['FPTP'],
-    'FVSP': ['FVSP'],
-    'FOYDP': ['FOYDP'],
-    'FYDR': ['FYDR'],
-    'FUV_1': ['FUV'],
-    'FUV_2': ['FUV'],
-    'FUV_3': ['FUV'],
-    'FDV_1': ['FDV'],
-    'FDV_2': ['FDV'],
-    'FDV_3': ['FDV'],
-    'DWTT1': ['DWTT'],
-    'DWTT2': ['DWTT'],
-    'DVC1': ['DVC'],
-    'DVC2': ['DVC'],
-    'DVC3': ['DVC'],
-    'NB1': ['NB_'],
-    'NB2': ['NB_'],
-    'FPT05': ['FPT05'],
-    'FPTP05': ['FPTP05']
-}
-
-metrics2ids = {
-    'R2': 1,
-    'MAE': 2,
-    'MAE train': 3,
-    'MAE test': 4,
-    'P95': 5,
-    'Samples': 6
-}
-
-targets2db = {
-    'FPT': 'FPT',
-    'FVS': 'FVS',
-    'FOYD': 'FOYD',
-    'FOS': 'FOS',
-    'FPTP': 'FPTP',
-    'FVSP': 'FVSP',
-    'FOYDP': 'FOYDP',
-    'FYDR': 'FYDR',
-    'FUV': 'FUV_1',
-    'FDV': 'FDV_1',
-    'DWTT': 'DWTT1',
-    'DVC': 'DVC1',
-    'NB_': 'NB1',
-    'FPT05': 'FPT05',
-    'FPTP05': 'FPTP05'
-}
-
+cx.init_oracle_client(lib_dir= r'C:/Program Files/Oracle/instantclient_21_3')
 
 class DB_Connector:
     def __init__(self,
-                 server: str = '',
-                 user: str = '',
-                 password: str = '',
-                 db: str = ''):
+                 server: str = '', user: str = '', password: str = '', db: str = '',
+                 user_oracle = '', password_oracle = '', dsn_oracle = ''):
         self.conn = pymssql.connect(server, user, password, db)
         self.cursor = self.conn.cursor(as_dict=True)
 
+        # cx.init_oracle_client(lib_dir= r'C:/Program Files/Oracle/instantclient_21_3')
+        self.cx_conn = cx.connect(user=user_oracle, password=password_oracle, dsn=dsn_oracle)
+        self.cx_cursor = self.cx_conn.cursor()
+
+#######################################
+# Functions that work with the database that stores the configurations and results of models
+#######################################
+
     def get_columns_id(self, columns_names: list[str]) -> pd.DataFrame:
-        query = '''select column_id, name from Columns where '''
+        query = '''select column_id from Columns where '''
         for name in columns_names:
-            if name in targets2db:
-                query += f'(Columns.name = \'{targets2db[name]}\') or '
-            else:
-                query += f'(Columns.name = \'{name}\') or '
+            query += f'(Columns.name = \'{name}\') or '
         # erase extra ' or '
         query = query[:-4]
 
@@ -80,6 +34,12 @@ class DB_Connector:
 
         return df
 
+    def get_target_id(self, target:str) -> int:
+        query = f'select target_id from Targets where name=\'{target}\''
+        df = read_sql_query(query, self.conn)
+
+        return df.iloc[0][0]
+
     def update_status(self, model_id: int):
         query = '''update Models set status = 1
                    where model_id = %s'''
@@ -96,49 +56,60 @@ class DB_Connector:
             self.cursor.execute(query, (model_id, column_id))
         self.conn.commit()
 
-    def insert_metrics(self, model_id,
-                       metrics: dict[str, dict[str, str]]):
-        targets = list(metrics.keys())
-        df = self.get_columns_id(targets)
+    def insert_metrics(self, model_id:int,
+                       metrics: dict[str, dict[str, str]],
+                       target_id:int):
 
-        for index, row in df.iterrows():
-            column_name, column_id = row['name'], row['column_id']
-            for metric_name, metric_value in metrics[columns2targets[column_name][0]].items():
+        df = read_sql_query('select * from Metrics', self.conn)
+        metrics2ids = {i[1]:i[0] for i in df.to_numpy()}
+
+        for metric_name, metric_value in metrics.items():
 
                 query = '''insert into
-                           MetricToModels(model_id, target_id,
-                           metric_id, value)
-                           values (%s, %s, %s, %s)'''
-                self.cursor.execute(query, (model_id, column_id,
+                        MetricToModels(model_id, target_id,
+                        metric_id, value)
+                        values (%s, %s, %s, %s)'''
+                self.cursor.execute(query, (model_id, target_id,
                                             metrics2ids[metric_name],
                                             metric_value))
         self.conn.commit()
 
+    def get_targets(self, model_id:int) -> list[str]:
+        model_id = (model_id, )
+
+        query = """select name
+                        from TargetsToModel as T1
+                        inner join Targets as T2
+                        on T1.target_id = T2.target_id
+                        where T2.model_id=%s"""
+        self.cursor.execute(query, (model_id))
+        rows = self.cursor.fetchall()
+
+        targets = [i['name'] for i in rows]
+        return targets
+
     def get_columns(self, model_id: int) -> tuple[list, list]:
         def flatten(array: list[list[str]]) -> list[str]:
             return [item for sublist in array for item in sublist]
 
         model_id = (model_id, )
 
-        query = """select name, type
+        query = """select name
                         from ColumnsToModels as T1
                         inner join Columns as T2
                         on T1.column_id = T2.column_id
-                        where T1.model_id=%s"""
+                        where T1.model_id=%s
+                        and T2.type=0"""
         self.cursor.execute(query, (model_id))
 
         rows = self.cursor.fetchall()
 
-        target_columns = [i['name'] for i in rows if i['type'] is True]
         feature_columns = [i['name'] for i in rows if i['type'] is False]
+        target_columns = self.get_targets(model_id)
 
         if not target_columns:
             raise Exception('''No targets have been selected,
                             fitting is impossible.\n''')
-        # ex: ['FPT', 'FUV_1'] -> ['FPT', 'FUV']
-        else:
-            target_columns = [columns2targets[i] for i in target_columns]
-            target_columns = list(set(flatten(target_columns)))
 
         return target_columns, feature_columns
 
@@ -199,42 +170,60 @@ class DB_Connector:
 
         return self._create_where(filters)
 
-    def _create_where_query(self, model_id: int) -> str:
+#######################################
+# Functions that work with the integrolayer database; /DataModel was used instead
+#######################################
+
+    def _create_where_query(self, model_id: int, TABLENAME:str=None, number_of_last_months:int=None) -> str:
         where_for_num_query = self._create_where_for_num_filters(model_id)
         where_for_cat_query = self._create_where_for_cat_filters(model_id)
 
-        query = 'select * from Data_Model'
+        # if TABLENAME:
+        query = f'select * from {TABLENAME} '
         if where_for_cat_query != '' and where_for_num_query == '':
             query = query + ' where ' + where_for_cat_query
         elif where_for_num_query != '' and where_for_cat_query == '':
             query = query + ' where ' + where_for_num_query
         elif where_for_num_query != '' and where_for_cat_query != '':
             query = (query + ' where ' + where_for_num_query
-                           + ' and ' + where_for_cat_query)
+                        + ' and ' + where_for_cat_query)
+        return query
+
+        # query = 'select * from ( ' + query_before_group + add_where_for_last_months(number_of_last_months) + query_with_group
+
+        # if where_for_cat_query != '' and where_for_num_query == '':
+        #     query = query + ') where ' + where_for_cat_query
+        # elif where_for_num_query != '' and where_for_cat_query == '':
+        #     query = query + ') where ' + where_for_num_query
+        # elif where_for_num_query != '' and where_for_cat_query != '':
+        #     query = (query + ') where ' + where_for_num_query
+        #                    + ' and ' + where_for_cat_query)
+        # else:
+        #     query += ')'
 
         return query
 
-    def get_filtered_data(self, model_id: int) -> pd.DataFrame:
+    def get_filtered_data(self, model_id: int, number_of_last_months: int = 6) -> pd.DataFrame:
 
-        query = self._create_where_query(model_id)
-        self.cursor.execute(query)
-        data = self.cursor.fetchall()
-        df = pd.DataFrame(data)
+        query = self._create_where_query(model_id, 'Data')
+        df = read_sql_query(query, self.cx_conn)
 
         return df
 
+    # TO DO: to rewrite
     def get_data(self, data_id: int) -> pd.DataFrame:
-        query = 'select * from Data_Model where id=%s'
+        query = 'select * from Data  where id=%s'
         self.cursor.execute(query, (data_id))
         data = self.cursor.fetchall()
         return pd.DataFrame(data)
 
+    # TO DO: to rewrite
     def get_reliability_and_data(self,
                                  model_id: int,
                                  data_id: int
                                  ) -> tuple[bool, pd.DataFrame]:
 
-        query = self._create_where_query(model_id) + ' and ID=%s'
+        query = self._create_where_query(model_id, 'Data') + ' and ID=%s'
         self.cursor.execute(query, (data_id))
         data = self.cursor.fetchall()
         is_reliable = bool(pd.DataFrame(data).shape[0])
@@ -245,15 +234,19 @@ class DB_Connector:
 
 
 if __name__=='__main__':
-    USERNAME = ''
-    PASSWORD = ''
-    HOST = ''
-    DB = ''
+
+    USERNAME = 'u1359913_admin'
+    PASSWORD = '2xFs0*1s'
+    HOST = '31.31.196.202'
+    DB = 'u1359913_mechpredic'
+    user = 'READER'
+    password = 'GdQBEY8N'
+    dsn = '172.20.20.46:1521/OPTIM'
 
     model_id = 51
     data_id = 5
 
-    dbm = DB_Connector(HOST, USERNAME, PASSWORD, DB)
-    is_reliable, data = dbm.get_reliability_and_data(model_id, data_id)
+    dbm = DB_Connector(HOST, USERNAME, PASSWORD, DB, user, password, dsn)
+    # is_reliable, data = dbm.get_reliability_and_data(model_id, data_id)
 
-    print(data, '\n', is_reliable)
+    # print(data, '\n', is_reliable)
